/* 
    kirc -- KDE Infrared Remote Control

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <kirc.h>

// sockets for IRC connetiont to lirc
#include <netinet/in.h>                /* For struct sockaddr_in */
#include <errno.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>

#include <qpushbutton.h> 
#include <qlabel.h>

extern int h_errno;
extern int errno;

#ifdef myDEBUG
#define kDEBUG
#endif

// =================================================================================================

KIRCWidget::KIRCWidget( KConfig* pconf, unsigned short int _port )
  : aCodeDict(37), aWidgetDict(11), port(_port)
{
  if (!pconf)
    fatal("kirc: Null KConfig object");
  pConfig = pconf;
  aAvailableId = 1;
  aCodeDict.setAutoDelete(TRUE);
  aWidgetDict.setAutoDelete(TRUE);
  keyUP=KCodeEntry::normal;
  num10=0;

  if ( !open_socket_to_irc( port ) ) {
    warning("kirc: unable to open socket connection to lircmd!");
    warning("kric: infrared remote control funtionality will be disabled!");
  }  
#ifdef kDEBUG
  else
    debug("kirc: socket connection to port %d established",_port);
#endif

  configMode=false;

  kc= new KIRCConfig( &aCodeDict );
  kc->hide();

  connect( kc, SIGNAL(apply()), this, SLOT( applySLOT() ));
  connect( kc, SIGNAL(defaults()), this, SLOT( defaultsSLOT() ));
  connect( kc, SIGNAL(cancel()), this, SLOT( cancelSLOT() ));
  connect( this, SIGNAL( configDataReceivedSIGNAL( ullong )), kc, SLOT(receiveCodeSLOT( ullong )));

}

// =================================================================================================

KIRCWidget::~KIRCWidget()
{
  sync();

  /* remove all widgetEntries */
  aWidgetDict.clear();
  aCodeDict.clear();

  close_socket_to_irc();
}

// =================================================================================================

void KIRCWidget::applySLOT()
{
  kc->hide();
  configMode=false;

  QDictIterator<KCodeEntry> aKeyIt(aCodeDict); 
  aKeyIt.toFirst();
  while ( aKeyIt.current() ) {
    aKeyIt.current()->aCurrentKeyCode=aKeyIt.current()->aConfigKeyCode;
    ++aKeyIt;
  }

  sync();

}

// =================================================================================================

void KIRCWidget::defaultsSLOT()
{
  kc->hide();
  configMode=false;
}

// =================================================================================================

void KIRCWidget::cancelSLOT()
{
  kc->hide();
  configMode=false;
}

// =================================================================================================

bool KIRCWidget::open_socket_to_irc( unsigned short int _port )
{
  struct hostent *he;
  char hname[1024];
             
  if ( -1 == gethostname(hname,sizeof(hname))) {
    warning("kirc: unable to get hostname!");
    return false;
  }
    
#ifdef LIRC_VERSION_036
  struct sockaddr_in sa;    // server adress, lirc 0.3.6 uses TCP sockets
  sa.sin_family=AF_INET;
  sa.sin_port=htons(_port);
  sa.sin_addr.s_addr = inet_addr(hname);

  if (sa.sin_addr.s_addr == (int) -1) {
    if (!(he = gethostbyname(hname))) {
      warning("kirc: unable to get host by name!");
      return false;
    } else
      sa.sin_addr.s_addr = ((struct in_addr*)(he->h_addr))->s_addr;
  }

  sockfd = socket( AF_INET, SOCK_STREAM, 0 ); 
  if ( sockfd == -1 ) {
    warning("kirc: unable to create socket!");
    warning("kirc: port: %d",_port);
    warning("kric: host: %s",hname);
    return false;
  }
#else 
  // lirc 0.4.1 uses UNIX sockets

#endif

  // make the socket non-blocking here, usually using fcntl( O_NONBLOCK )
  if ( -1 == ::connect( sockfd, (struct sockaddr*)&sa, sizeof(sa) ) ) {
    warning("kirc: unable to connect to socket!");
    warning("kirc: error %d occured!",errno);
    return false;
  }
  
  sn = new QSocketNotifier( sockfd, QSocketNotifier::Read, this );
  QObject::connect( sn, SIGNAL(activated(int)),
		    this, SLOT(dataReceivedSLOT(int)) );

  if ( !sn->isEnabled() ) {
    warning("kirc: unable to open socket notifier!");
    return false;
  }

  return true;
}

// =================================================================================================

bool KIRCWidget::close_socket_to_irc()
{
#ifdef kDEBUG
  debug("kirc: closing socket connection to port %d",port);
#endif
  
  close(sockfd); 

  return true;
}

// =================================================================================================

void KIRCWidget::dataReceivedSLOT(int socket)
{

  if ( read(socket,buf,8) < 8 ) 
    warning("kirc: error reading data from socket!");

  data=ullong(buf[0])|(ullong(buf[1])<<8)|
    (ullong(buf[2])<<16)|(ullong(buf[3])<<24)|
    (ullong(buf[4])<<32)|(ullong(buf[5])<<40)|
    (ullong(buf[6])<<48)|(ullong(buf[7])<<56);

  if (configMode) {
    emit configDataReceivedSIGNAL( data );
    return;
  }

  QDictIterator<KCodeEntry> aKeyIt(aCodeDict);
  aKeyIt.toFirst();
  while ( aKeyIt.current() && !(aKeyIt.current()->aCurrentKeyCode == data) ) {
    ++aKeyIt;
  }

#ifdef kDEBUG
  debug("kirc: data: %d, 0x%x, function: <%s>",(int)data,(int)data,
	aKeyIt.current()?aKeyIt.currentKey():"<none>");
#endif

  if ( aKeyIt.current() && aKeyIt.current()->isEnabled ) {
    KCodeEntry *code= aKeyIt.current();

    switch ( code->typeOfCode ) {

    case KCodeEntry::twoDigits:
    case KCodeEntry::up10:
    case KCodeEntry::up20:
    case KCodeEntry::up30:
      keyUP= code->typeOfCode;
      num10=0;
      emit specialKeyReceivedSIGNAL( code->typeOfCode );
      return;

    case KCodeEntry::number:
      switch ( keyUP ) {
      case KCodeEntry::twoDigits:
	if ( num10 == 0 ) {
	  num10= QString( aKeyIt.currentKey() ).toInt();
	  return;
	} else {
	  num= QString( aKeyIt.currentKey() ).toInt() + num10 * 10;
	  keyUP=KCodeEntry::normal;
	}
      case KCodeEntry::up10:
	num= QString( aKeyIt.currentKey() ).toInt() + 10;
	keyUP=KCodeEntry::normal;
	break;
      case KCodeEntry::up20:
	num= QString( aKeyIt.currentKey() ).toInt() + 20;
	keyUP=KCodeEntry::normal;
	break;
      case KCodeEntry::up30:
	num= QString( aKeyIt.currentKey() ).toInt() + 30;
	keyUP=KCodeEntry::normal;
	break;
      case KCodeEntry::normal:
	num= QString( aKeyIt.currentKey() ).toInt();
	break;
      }

      emit numKeyReceivedSIGNAL( num );
      
#ifdef kDEBUG
      debug("kirc: numKeyReceivedSIGNAL( %d )", num );
#endif
      break;

    case KCodeEntry::normal:
      QDictIterator<KCodeConnectEntry> kcce(*code->pConnectDict);
      kcce.toFirst();

      if ( kcce.current() ) {
#ifdef kDEBUG
	debug("kirc: call function");
#endif
	kcce.current()->sig->activate();
      }
      
      break;
    } // switch
 
  }
}

// =================================================================================================

void KIRCWidget::reconnectToSocket( unsigned short int _port )
{
  if ( !close_socket_to_irc() ) {
    warning("kirc: ...");
    return;
  }
  if ( open_socket_to_irc( _port ) ) port= _port;
  else warning("kirc: ");
}

// =================================================================================================

bool KIRCWidget::addCode( const QString& functionName, uint defaultCode, 
			  KCodeEntry::codeType type, bool configurable )
{
#ifdef kDEBUG
  debug("kirc: register function %s",(const char *)functionName);
#endif
  /* search for an existing "functionName" entry */
  KCodeEntry *pEntry = aCodeDict[ functionName ];
	
  if ( pEntry ) /* already exists : remove it */
    removeCode(functionName);

  pEntry = new KCodeEntry;
  aCodeDict.insert( functionName, pEntry );
	
  pEntry->aDefaultKeyCode = defaultCode;
  pEntry->aCurrentKeyCode = defaultCode;
  pEntry->bConfigurable = configurable;
  pEntry->aAccelId = 0;
  pEntry->typeOfCode= type;
  pEntry->pConnectDict = NULL;
  pEntry->isEnabled= true;

  if ( !configurable )
    return TRUE;

	/* search an entry in the KConfig object */
  pConfig->setGroup("irc_codes");
  if ( !pConfig->hasKey(functionName) )
    return TRUE;
	
	/* read and recognize the key */
  ullong aKeyCode = (ullong) pConfig->readUnsignedLongNumEntry(functionName);
  if ( aKeyCode ) /* if recognized */
    pEntry->aCurrentKeyCode = aKeyCode;
	
  return TRUE;
}

// =================================================================================================

void KIRCWidget::removeCode( const QString& functionName )
{
  /* search for an existing "functionName" entry */
  KCodeEntry *pEntry = aCodeDict[ functionName ];
	
  if ( !pEntry ) 
    return;
	
  /* disconnect eventuals connections */
  if ( pEntry->aAccelId ) {
    QDictIterator<KCodeConnectEntry> aConnectIt(*pEntry->pConnectDict);
    aConnectIt.toFirst();
    while ( aConnectIt.current() ) {
      disconnectFunction( aConnectIt.currentKey(), functionName );
      ++aConnectIt;
    }
  }
	
  aCodeDict.remove( functionName );
}

// =================================================================================================

unsigned int KIRCWidget::readCurrentKey( const QString& functionName )
{
  KCodeEntry *pEntry = aCodeDict[ functionName ];
	
  if ( !pEntry )
    return 0;
  else
    return pEntry->aCurrentKeyCode;
}

// =================================================================================================

unsigned int KIRCWidget::readDefaultKey( const QString& functionName )
{
  KCodeEntry *pEntry = aCodeDict[ functionName ];
	
  if ( !pEntry )
    return 0;
  else
    return pEntry->aDefaultKeyCode;
}

// =================================================================================================

void KIRCWidget::registerWidget( const QString& widgetName, QWidget* currentWidget )
{
#ifdef kDEBUG
  debug("kirc: register widget %s",(const char *)widgetName);
#endif

  KIRCWidgetEntry *pWEntry = aWidgetDict[ widgetName ];
  if ( pWEntry ) {
    QString str;
    str.sprintf( "initKeyWidget : \"%s\" widget already initialized",
		 (const char *)widgetName );
    warning(str);
    return;
  }

  pWEntry = new KIRCWidgetEntry;
  aWidgetDict.insert( widgetName, pWEntry );
  pWEntry->sWidgetName= widgetName;
  pWEntry->pWidget = currentWidget;

}
// =================================================================================================

void KIRCWidget::connectFunction( const QString& widgetName, 
				  const QString& functionName,
				  const QObject* receiver, const char* member,
				  bool activate )
{
#ifdef kDEBUG
  //debug("kirc: connect %s -> %s",(const char *)functionName,member);
#endif
  QString str;
	
  /* search for an existing "widgetName" entry */
  KIRCWidgetEntry *pWEntry = aWidgetDict[ widgetName ];
  if ( !pWEntry ) {
    str.sprintf( "connectFunction : \"%s\" widget has not been initialized",
		 (const char *)widgetName );
    warning(str);
    return;
  }

  /* search for an existing "functionName" entry */
  KCodeEntry *pEntry = aCodeDict[ functionName ];
  if ( !pEntry ) {
    str.sprintf( "connectFunction : \"%s\" function does not exist",
		 (const char *)functionName );
    warning(str);
    return;
  }

  if ( pEntry->typeOfCode != KCodeEntry::normal ) {
    str.sprintf( "connectFunction : a special key cannnot be connected",
		 (const char *)functionName );
    warning(str);
    return;
  }
	
  KCodeConnectEntry *pCEntry;
	
  /* if the widget is already connected */
  if ( pEntry->pConnectDict ) {
    pCEntry = (*pEntry->pConnectDict)[ widgetName ];
    if ( pCEntry )
      //      internalDisconnectFunction( widgetName, pWEntry, pEntry, pCEntry );
      pEntry->pConnectDict->remove( widgetName );
  } else {
    pEntry->pConnectDict = new QDict<KCodeConnectEntry>(13);
    pEntry->pConnectDict->setAutoDelete(TRUE);
    pEntry->aAccelId = aAvailableId;
    aAvailableId++;
  }
		
  pCEntry = new KCodeConnectEntry;
  pCEntry->pReceiver = (QObject *)receiver;
  pCEntry->sMember = member;
  pCEntry->sig= new QSignal;
  pCEntry->sig->connect( receiver, member);
  pEntry->pConnectDict->insert( widgetName, pCEntry );

  /*	
  pWEntry->createItem( pEntry->aAccelId, pEntry->aCurrentKeyCode,
		       (QObject *)receiver, member );
		       */

  if ( !activate )
    toggleFunction( widgetName, functionName, FALSE );

}

// =================================================================================================

void KIRCWidget::toggleFunction( const QString& widgetName,const QString& functionName, bool activate )
{	
  QString str;
	
  /* search for an existing "widgetName" entry */
  KIRCWidgetEntry *pWEntry = aWidgetDict[ widgetName ];
  if ( !pWEntry ) {
    str.sprintf( "(dis)activateFunction : \"%s\" widget has not been initialized",
		 (const char *)widgetName );
    warning(str);
    return;
  }
	
  /* search for an existing "functionName" entry */
  KCodeEntry *pEntry = aCodeDict[ functionName ];
  if ( !pEntry ) {
    str.sprintf( "(dis)activateFunction : \"%s\" function does not exist",
		 (const char *)functionName );
    warning(str);
    return;
  }

  pEntry->isEnabled= activate;
}

// =================================================================================================

void KIRCWidget::disconnectFunction( const QString& widgetName,
				     const QString& functionName )
{
  /* search for an existing "widgetName" entry */
  KIRCWidgetEntry *pWEntry = aWidgetDict[ widgetName ];
  if ( !pWEntry ) {
    QString str;
    str.sprintf( "disconnectFunction : \"%s\" widget has not been initialized",
		 (const char *)widgetName );
    warning(str);
    return;
  }
	
  /* search for an existing "functionName" entry */
  KCodeEntry *pEntry = aCodeDict[ functionName ];
  if ( !pEntry ) 
    return;
  /* search for a connection */
  KCodeConnectEntry *pCEntry = (*pEntry->pConnectDict)[ widgetName ];
  if ( !pCEntry ) 
    return;
  pCEntry->sig->disconnect( pCEntry->pReceiver, pCEntry->sMember );
  delete pCEntry->sig;

  //internalDisconnectFunction( widgetName, pWEntry, pEntry, pCEntry );
  // anstatt
  pEntry->pConnectDict->remove( widgetName );

  /* if this was the unique connection of the functionName : delete the
	   pConnectDict */
  if ( pEntry->pConnectDict->count()==0 )
    delete pEntry->pConnectDict;
}

// =================================================================================================
/*
void KIRCWidget::internalDisconnectFunction( const QString& widgetName, 
					     KIRCWidgetEntry *pWEntry, KCodeEntry *pEntry, KCodeConnectEntry *pCEntry )
{
  pWEntry->deleteItem( pEntry->aAccelId, pCEntry->pReceiver,
		       pCEntry->sMember );
  pEntry->pConnectDict->remove( widgetName );
}
*/
// =================================================================================================

void KIRCWidget::disconnectAllFunctions( const QString& widgetName )
{
  QDictIterator<KCodeEntry> aKeyIt(aCodeDict);
  aKeyIt.toFirst();
  while ( aKeyIt.current() ) {
    disconnectFunction( widgetName, aKeyIt.currentKey() );
    ++aKeyIt;
  }
}

// =================================================================================================
void KIRCWidget::destroyWidgetEntry( const QString& widgetName )
{
  aWidgetDict.remove( widgetName );
}
// =================================================================================================

void KIRCWidget::internalDisconnectAll( const QString& widgetName)
{
  QDictIterator<KCodeEntry> aKeyIt(aCodeDict); 
  aKeyIt.toFirst();
  while ( aKeyIt.current() ) {
    if ( aKeyIt.current()->pConnectDict )
      if ( (*aKeyIt.current()->pConnectDict)[ widgetName ] )
	aKeyIt.current()->pConnectDict->remove( widgetName );
    ++aKeyIt;
  }
}

// =================================================================================================

void KIRCWidget::sync()
{
  /* write the current values in aCodeDict to KConfig object */
  pConfig->setGroup("irc_codes");

  QDictIterator<KCodeEntry> aKeyIt(aCodeDict);
  aKeyIt.toFirst();
  while ( aKeyIt.current() ) {
    if ( aKeyIt.current()->bConfigurable )
      pConfig->writeEntry( aKeyIt.currentKey(),(unsigned long) aKeyIt.current()->aCurrentKeyCode );
    ++aKeyIt;
  }
}

// =================================================================================================

bool KIRCWidget::configureCodes( QWidget *parent )
{
#ifdef kDEBUG
  debug("kirc: Config start ");
#endif
  
  QDictIterator<KCodeEntry> aKeyIt(aCodeDict); 
  aKeyIt.toFirst();
  while ( aKeyIt.current() ) {
    aKeyIt.current()->aConfigKeyCode=aKeyIt.current()->aCurrentKeyCode;
    ++aKeyIt;
  }

  kc->newDictSLOT( &aCodeDict );
  kc->show();
  configMode=true;

#ifdef kDEBUG
  debug("kirc: Config end ");
#endif
  return true;
}

// =================================================================================================
// =================================================================================================

KIRCConfig::KIRCConfig(QDict<KCodeEntry> *_aCodeDict, QWidget * parent = 0, const char * name = 0 )
  : aCodeDict(_aCodeDict), QWidget( parent, name )
{
#ifdef kDEBUG
  debug("kircConfig: KIRCConfig");
#endif
  setCaption("Infrared Remote Control -- Code Setup");

  lv= new QListView( this );
  lv->addColumn("Function Name");
  lv->setColumnWidthMode(0,QListView::Manual);
  lv->setColumnWidth(0,195);
  lv->addColumn("IRC Button Code");
  lv->setColumnWidthMode(1,QListView::Manual);
  lv->setColumnWidth(1,175);
  //addColumn("Connectet To");

  lv->setFrameStyle( 51 );
  lv->setLineWidth( 2 );
  lv->setMultiSelection ( false );
  lv->setAllColumnsShowFocus( true );

  newDictSLOT( _aCodeDict );

  resize(400,300);
  setMinimumSize( 400,300 );
  setMaximumSize( 400,300 );

  lv->setGeometry(5,5,390,200);
  connect( lv, SIGNAL( selectionChanged ( QListViewItem * ) ),this, SLOT(selectionChangedSLOT( QListViewItem * )) );

  QFrame* dlgedit_Frame_1;
  dlgedit_Frame_1 = new QFrame( this, "Frame_1" );
  dlgedit_Frame_1->setGeometry( 5, height() - 80 - 10, 390, 5 );
  dlgedit_Frame_1->setMinimumSize( 10, 10 );
  dlgedit_Frame_1->setMaximumSize( 32767, 32767 );
  dlgedit_Frame_1->setFrameStyle( 52 );

  le= new QLineEdit( this );
  le->setGeometry( 230,height() - 80 , 150 ,30);
  connect( le, SIGNAL(textChanged ( const char * ) ), this, SLOT(textChangedSLOT( const char *)));

  QLabel *label= new QLabel( "Enter IRC code or press IRC button:", this );
  label->setGeometry( 20 ,height() - 80 , 200 ,30);
  
  QFrame* dlgedit_Frame_2;
  dlgedit_Frame_2 = new QFrame( this, "Frame_1" );
  dlgedit_Frame_2->setGeometry( 5, height() - 50 , 390, 5 );
  dlgedit_Frame_2->setMinimumSize( 10, 10 );
  dlgedit_Frame_2->setMaximumSize( 32767, 32767 );
  dlgedit_Frame_2->setFrameStyle( 52 );

  QPushButton *b1= new QPushButton( "OK", this );
  b1->setGeometry(  width()-320, height()-30, 100, 25);
  connect( b1,SIGNAL( clicked () ), this, SLOT( applySLOT() ));

  QPushButton *b2= new QPushButton( "Default", this );
  b2->setGeometry(  width()-210,height()-30, 100, 25);
  connect( b2,SIGNAL( clicked () ), this, SLOT( defaultsSLOT() ));

  QPushButton *b3= new QPushButton( "Cancel", this );
  b3->setGeometry( width()-100, height()-30, 90, 25);
  connect( b3,SIGNAL( clicked () ), this, SLOT( cancelSLOT() ));

  newDictInsert=false;
}

// =================================================================================================

KIRCConfig::~KIRCConfig()
{
  delete lv;
}

// =================================================================================================

void KIRCConfig::newDictSLOT( QDict<KCodeEntry> *_aCodeDict )
{
#ifdef kDEBUG
  debug("kircConfig: newDict begin");
#endif

  newDictInsert=true;

  lv->clear();
  aCodeDict= _aCodeDict;
  QDictIterator<KCodeEntry> aKeyIt(*aCodeDict);
  aKeyIt.toFirst();
  while ( aKeyIt.current() ) {
    QString code;
    code.sprintf("dec: %d, hex: 0x%x", 
		 (long unsigned int) aKeyIt.current()->aConfigKeyCode,
		 (long unsigned int) aKeyIt.current()->aConfigKeyCode);
    QString key(aKeyIt.currentKey());
    if ( aKeyIt.current()->typeOfCode == KCodeEntry::number )
      key= QString("number key \"")+key+QString("\"");
    if ( ( aKeyIt.current()->typeOfCode == KCodeEntry::twoDigits ) ||
	 ( aKeyIt.current()->typeOfCode == KCodeEntry::up10 ) ||
	 ( aKeyIt.current()->typeOfCode == KCodeEntry::up20 ) ||
	 ( aKeyIt.current()->typeOfCode == KCodeEntry::up30 ) )
      key= QString("special key \"")+key+QString("\"");
    (void) new QListViewItem( lv, (const char*)key, (const char*)code );
    ++aKeyIt;
  }

#ifdef kDEBUG
  debug("kircConfig: newDict end");
#endif

  newDictInsert=false;

  currentItem= lv->firstChild ();
  lv->setSelected ( currentItem , true );

  aKeyIt.toFirst();
  currentCode= aKeyIt.current();

#ifdef kDEBUG
  debug("kircConfig: newDict ready");
#endif
}

// =================================================================================================

void KIRCConfig::applySLOT()
{
  emit apply();
}

// =================================================================================================

void KIRCConfig::defaultsSLOT()
{
#ifdef kDEBUG
  debug("kircConfig: default");
#endif
  currentCode->aConfigKeyCode=currentCode->aCurrentKeyCode;

  QString text;
  text.setNum( (long unsigned int) currentCode->aConfigKeyCode );
  le->setText( (const char*) text );
  text.sprintf("dec: %d, hex: 0x%x", 
	       (long unsigned int) currentCode->aConfigKeyCode, 
	       (long unsigned int) currentCode->aConfigKeyCode );
  currentItem->setText( 1, text );
}

// =================================================================================================

void KIRCConfig::cancelSLOT()
{
  emit cancel();
}

// =================================================================================================

void KIRCConfig::receiveCodeSLOT( ullong code )
{
#ifdef kDEBUG
  debug("kircConfig: data: %d, 0x%x",(int)code,(int)code);
#endif
  currentCode->aConfigKeyCode= code;

  QString text;
  text.setNum( (long unsigned int) code);

  le->setText( (const char*) text );
  //currentItem->setText( 1, (const char*) text );
  
}

// =================================================================================================

void KIRCConfig::selectionChangedSLOT( QListViewItem * item )
{
  if (newDictInsert) return;

#ifdef kDEBUG
  debug("kircConfig: select <%s>",item->text(0));
#endif

  currentItem= item;
  QString sitem( item->text(0) );
  if ( -1 != sitem.find('\"')) {
    sitem.remove(0,sitem.find('\"')+1);
    sitem.remove(sitem.find('\"'),sitem.length()-sitem.find('\"'));
    currentCode= (*aCodeDict)[ (const char*) sitem ];
  } else {
    currentCode= (*aCodeDict)[ (const char*) item->text(0) ];
  }
  QString tmp;
  tmp.setNum( (long unsigned int) currentCode->aConfigKeyCode );
  le->setText( (const char*) tmp );
}

// =================================================================================================

void KIRCConfig::textChangedSLOT( const char * text)
{
  currentCode->aConfigKeyCode=atol( text );
  QString code;
  code.sprintf("dec: %d, hex: 0x%x", 
	       (long unsigned int) currentCode->aConfigKeyCode,
	       (long unsigned int) currentCode->aConfigKeyCode);
  currentItem->setText( 1, code );
}

// =================================================================================================
// =================================================================================================

#include "kirc.moc"




